@@ -187,6 +187,9 @@ const AllView = new Lang.Class({
     _init: function() {
         this.parent();
 
+        this._grid.actor.y_align = Clutter.ActorAlign.START;
+        this._grid.actor.y_expand = true;
+
         let box = new St.BoxLayout({ vertical: true });
         this._stack = new St.Widget({ layout_manager: new AllViewLayout() });
         this._stack.add_actor(this._grid.actor);
@@ -276,8 +279,12 @@ const AllView = new Lang.Class({
                 this._eventBlocker.reactive = isOpen;
                 this._currentPopup = isOpen ? popup : null;
                 this._updateIconOpacities(isOpen);
-                if (isOpen)
+                if (isOpen) {
                     this._ensureIconVisible(popup.actor);
+                    this._grid.actor.y = popup.parentOffset;
+                } else {
+                    this._grid.actor.y = 0;
+                }
             }));
     },
 
@@ -343,6 +350,9 @@ const AppDisplay = new Lang.Class({
         global.settings.connect('changed::app-folder-categories', Lang.bind(this, function() {
             Main.queueDeferredWork(this._allAppsWorkId);
         }));
+        this._privacySettings = new Gio.Settings({ schema: 'org.gnome.desktop.privacy' });
+        this._privacySettings.connect('changed::remember-app-usage',
+                                      Lang.bind(this, this._updateFrequentVisibility));
 
         this._views = [];
 
@@ -386,6 +396,7 @@ const AppDisplay = new Lang.Class({
                 }));
         }
         this._showView(Views.FREQUENT);
+        this._updateFrequentVisibility();
 
         // We need a dummy actor to catch the keyboard focus if the
         // user Ctrl-Alt-Tabs here before the deferred work creates
@@ -415,6 +426,19 @@ const AppDisplay = new Lang.Class({
         }
     },
 
+    _updateFrequentVisibility: function() {
+        let enabled = this._privacySettings.get_boolean('remember-app-usage');
+        this._views[Views.FREQUENT].control.visible = enabled;
+
+        let visibleViews = this._views.filter(function(v) {
+            return v.control.visible;
+        });
+        this._controls.visible = visibleViews.length > 1;
+
+        if (!enabled && this._views[Views.FREQUENT].view.actor.visible)
+            this._showView(Views.ALL);
+    },
+
     _redisplay: function() {
         this._redisplayFrequentApps();
         this._redisplayAllApps();
@@ -574,7 +598,11 @@ const FolderIcon = new Lang.Class({
         // Position the popup above or below the source icon
         if (side == St.Side.BOTTOM) {
             this._popup.actor.show();
-            this._popup.actor.y = this.actor.y - this._popup.actor.height;
+            let closeButtonOffset = -this._popup.closeButton.translation_y;
+            let y = this.actor.y - this._popup.actor.height;
+            let yWithButton = y - closeButtonOffset;
+            this._popup.parentOffset = yWithButton < 0 ? -yWithButton : 0;
+            this._popup.actor.y = Math.max(y, closeButtonOffset);
             this._popup.actor.hide();
         } else {
             this._popup.actor.y = this.actor.y + this.actor.height;
@@ -597,6 +625,7 @@ const AppFolderPopup = new Lang.Class({
         this._arrowSide = side;
 
         this._isOpen = false;
+        this.parentOffset = 0;
 
         this.actor = new St.Widget({ layout_manager: new Clutter.BinLayout(),
                                      visible: false,
@@ -620,17 +649,31 @@ const AppFolderPopup = new Lang.Class({
         this.actor.add_actor(this._boxPointer.actor);
         this._boxPointer.bin.set_child(this._view.actor);
 
-        let closeButton = Util.makeCloseButton();
-        closeButton.connect('clicked', Lang.bind(this, this.popdown));
-        this.actor.add_actor(closeButton);
+        this.closeButton = Util.makeCloseButton();
+        this.closeButton.connect('clicked', Lang.bind(this, this.popdown));
+        this.actor.add_actor(this.closeButton);
 
-        this._boxPointer.actor.bind_property('opacity', closeButton, 'opacity',
+        this._boxPointer.actor.bind_property('opacity', this.closeButton, 'opacity',
                                              GObject.BindingFlags.SYNC_CREATE);
 
+        global.focus_manager.add_group(this.actor);
+
         source.actor.connect('destroy', Lang.bind(this,
             function() {
                 this.actor.destroy();
             }));
+        this.actor.connect('key-press-event', Lang.bind(this, this._onKeyPress));
+    },
+
+    _onKeyPress: function(actor, event) {
+        if (!this._isOpen)
+            return false;
+
+        if (event.get_key_symbol() != Clutter.KEY_Escape)
+            return false;
+
+        this.popdown();
+        return true;
     },
 
     toggle: function() {
@@ -645,6 +688,7 @@ const AppFolderPopup = new Lang.Class({
             return;
 
         this.actor.show();
+        this.actor.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false);
 
         this._boxPointer.setArrowActor(this._source.actor);
         this._boxPointer.show(BoxPointer.PopupAnimation.FADE |
