@@ -137,7 +137,6 @@ const Monitor = new Lang.Class({
 const defaultParams = {
     trackFullscreen: false,
     affectsStruts: false,
-    affectsInputRegion: true
 };
 
 const LayoutManager = new Lang.Class({
@@ -189,10 +188,12 @@ const LayoutManager = new Lang.Class({
         global.stage.remove_actor(global.window_group);
         this.uiGroup.add_actor(global.window_group);
 
-        global.stage.remove_actor(global.overlay_group);
-        this.uiGroup.add_actor(global.overlay_group);
         global.stage.add_child(this.uiGroup);
 
+        this.overviewGroup = new St.Widget({ name: 'overviewGroup',
+                                             visible: false });
+        this.addChrome(this.overviewGroup);
+
         this.screenShieldGroup = new St.Widget({ name: 'screenShieldGroup',
                                                  visible: false,
                                                  clip_to_allocation: true,
@@ -243,24 +244,24 @@ const LayoutManager = new Lang.Class({
         this._monitorsChanged();
     },
 
-    // This is called by Main after everything else is constructed;
-    // it needs access to Main.overview, which didn't exist
-    // yet when the LayoutManager was constructed.
+    // This is called by Main after everything else is constructed
     init: function() {
-        Main.overview.connect('showing', Lang.bind(this, this._overviewShowing));
-        Main.overview.connect('hidden', Lang.bind(this, this._overviewHidden));
         Main.sessionMode.connect('updated', Lang.bind(this, this._sessionUpdated));
 
-        this._prepareStartupAnimation();
+        this._loadBackground();
     },
 
-    _overviewShowing: function() {
+    showOverview: function() {
+        this.overviewGroup.show();
+
         this._inOverview = true;
         this._updateVisibility();
         this._queueUpdateRegions();
     },
 
-    _overviewHidden: function() {
+    hideOverview: function() {
+        this.overviewGroup.hide();
+
         this._inOverview = false;
         this._updateVisibility();
         this._queueUpdateRegions();
@@ -556,6 +557,25 @@ const LayoutManager = new Lang.Class({
         return this._keyboardIndex;
     },
 
+    _loadBackground: function() {
+        this._systemBackground = new Background.SystemBackground();
+        this._systemBackground.actor.hide();
+
+        global.stage.insert_child_below(this._systemBackground.actor, null);
+
+        let constraint = new Clutter.BindConstraint({ source: global.stage,
+                                                      coordinate: Clutter.BindCoordinate.ALL });
+        this._systemBackground.actor.add_constraint(constraint);
+
+        let signalId = this._systemBackground.connect('loaded', Lang.bind(this, function() {
+            this._systemBackground.disconnect(signalId);
+            this._systemBackground.actor.show();
+            global.stage.show();
+
+            this._prepareStartupAnimation();
+        }));
+    },
+
     // Startup Animations
     //
     // We have two different animations, depending on whether we're a greeter
@@ -601,35 +621,18 @@ const LayoutManager = new Lang.Class({
             global.window_group.set_clip(monitor.x, monitor.y, monitor.width, monitor.height);
         }
 
-        this._systemBackground = new Background.SystemBackground();
-        this._systemBackground.actor.hide();
+        this.emit('startup-prepared');
 
-        global.stage.insert_child_below(this._systemBackground.actor, null);
-
-        let constraint = new Clutter.BindConstraint({ source: global.stage,
-                                                      coordinate: Clutter.BindCoordinate.ALL });
-        this._systemBackground.actor.add_constraint(constraint);
-
-        let signalId = this._systemBackground.connect('loaded',
-                                                      Lang.bind(this, function() {
-                                                          this._systemBackground.disconnect(signalId);
-                                                          this._systemBackground.actor.show();
-                                                          global.stage.show();
-
-                                                          this.emit('startup-prepared');
-
-                                                          // We're mostly prepared for the startup animation
-                                                          // now, but since a lot is going on asynchronously
-                                                          // during startup, let's defer the startup animation
-                                                          // until the event loop is uncontended and idle.
-                                                          // This helps to prevent us from running the animation
-                                                          // when the system is bogged down
-                                                          GLib.idle_add(GLib.PRIORITY_LOW,
-                                                                        Lang.bind(this, function() {
-                                                                            this._startupAnimation();
-                                                                            return false;
-                                                                        }));
-                                                      }));
+        // We're mostly prepared for the startup animation
+        // now, but since a lot is going on asynchronously
+        // during startup, let's defer the startup animation
+        // until the event loop is uncontended and idle.
+        // This helps to prevent us from running the animation
+        // when the system is bogged down
+        GLib.idle_add(GLib.PRIORITY_LOW, Lang.bind(this, function() {
+            this._startupAnimation();
+            return false;
+        }));
     },
 
     _startupAnimation: function() {
@@ -686,7 +689,6 @@ const LayoutManager = new Lang.Class({
     },
 
     showKeyboard: function () {
-        this.keyboardBox.raise_top();
         Tweener.addTween(this.keyboardBox,
                          { anchor_y: this.keyboardBox.height,
                            time: KEYBOARD_ANIMATION_TIME,
@@ -731,11 +733,10 @@ const LayoutManager = new Lang.Class({
     // @actor: an actor to add to the chrome
     // @params: (optional) additional params
     //
-    // Adds @actor to the chrome, and (unless %affectsInputRegion in
-    // @params is %false) extends the input region to include it.
-    // Changes in @actor's size, position, and visibility will
-    // automatically result in appropriate changes to the input
-    // region.
+    // Adds @actor to the chrome, and extends the input region
+    // to include it. Changes in @actor's size, position, and
+    // visibility will automatically result in appropriate changes
+    // to the input region.
     //
     // If %affectsStruts in @params is %true (and @actor is along a
     // screen edge), then @actor's size and position will also affect
@@ -949,7 +950,7 @@ const LayoutManager = new Lang.Class({
 
         for (i = 0; i < this._trackedActors.length; i++) {
             let actorData = this._trackedActors[i];
-            if (!(actorData.affectsInputRegion && wantsInputRegion) && !actorData.affectsStruts)
+            if (!wantsInputRegion && !actorData.affectsStruts)
                 continue;
 
             let [x, y] = actorData.actor.get_transformed_position();
@@ -959,13 +960,8 @@ const LayoutManager = new Lang.Class({
             w = Math.round(w);
             h = Math.round(h);
 
-            if (actorData.affectsInputRegion && wantsInputRegion) {
-                let rect = new Meta.Rectangle({ x: x, y: y, width: w, height: h});
-
-                if (actorData.actor.get_paint_visibility() &&
-                    !this.uiGroup.get_skip_paint(actorData.actor))
-                    rects.push(rect);
-            }
+            if (wantsInputRegion && actorData.actor.get_paint_visibility())
+                rects.push(new Meta.Rectangle({ x: x, y: y, width: w, height: h }));
 
             if (actorData.affectsStruts) {
                 // Limit struts to the size of the screen
@@ -1136,11 +1132,11 @@ const HotCorner = new Lang.Class({
                                              height: 3,
                                              reactive: true });
 
-            this._corner = new Clutter.Rectangle({ name: 'hot-corner',
-                                                   width: 1,
-                                                   height: 1,
-                                                   opacity: 0,
-                                                   reactive: true });
+            this._corner = new Clutter.Actor({ name: 'hot-corner',
+                                               width: 1,
+                                               height: 1,
+                                               opacity: 0,
+                                               reactive: true });
             this._corner._delegate = this;
 
             this.actor.add_child(this._corner);
