@@ -107,7 +107,7 @@ const OsdWindow = new Lang.Class({
                                    Lang.bind(this, this._monitorsChanged));
         this._monitorsChanged();
 
-        Main.layoutManager.addChrome(this.actor, { affectsInputRegion: false });
+        Main.uiGroup.add_child(this.actor);
     },
 
     setIcon: function(icon) {
@@ -136,10 +136,8 @@ const OsdWindow = new Lang.Class({
             return;
 
         if (!this.actor.visible) {
-            Meta.disable_unredirect_for_screen(global.screen);
             this.actor.show();
             this.actor.opacity = 0;
-            this.actor.get_parent().set_child_above_sibling(this.actor, null);
 
             Tweener.addTween(this.actor,
                              { opacity: 255,
@@ -167,11 +165,7 @@ const OsdWindow = new Lang.Class({
                          { opacity: 0,
                            time: FADE_TIME,
                            transition: 'easeOutQuad',
-                           onComplete: Lang.bind(this, function() {
-                              this._reset();
-                              Meta.enable_unredirect_for_screen(global.screen);
-                           })
-                         });
+                           onComplete: Lang.bind(this, this._reset) });
     },
 
     _reset: function() {
