@@ -115,7 +115,7 @@ const Overview = new Lang.Class({
         let monitor = Main.layoutManager.primaryMonitor;
 
         this._desktopFade = new St.Bin();
-        global.overlay_group.add_actor(this._desktopFade);
+        Main.layoutManager.overviewGroup.add_child(this._desktopFade);
 
         let layout = new Clutter.BinLayout();
         this._stack = new Clutter.Actor({ layout_manager: layout });
@@ -132,8 +132,7 @@ const Overview = new Lang.Class({
         this._overview._delegate = this;
 
         this._backgroundGroup = new Meta.BackgroundGroup();
-        global.overlay_group.add_child(this._backgroundGroup);
-        this._backgroundGroup.hide();
+        Main.layoutManager.overviewGroup.add_child(this._backgroundGroup);
         this._bgManagers = [];
 
         this._activationTime = 0;
@@ -147,14 +146,13 @@ const Overview = new Lang.Class({
         // During transitions, we raise this to the top to avoid having the overview
         // area be reactive; it causes too many issues such as double clicks on
         // Dash elements, or mouseover handlers in the workspaces.
-        this._coverPane = new Clutter.Rectangle({ opacity: 0,
-                                                  reactive: true });
+        this._coverPane = new Clutter.Actor({ opacity: 0,
+                                              reactive: true });
         this._overview.add_actor(this._coverPane);
         this._coverPane.connect('event', Lang.bind(this, function (actor, event) { return true; }));
 
-        this._stack.hide();
         this._stack.add_actor(this._overview);
-        global.overlay_group.add_actor(this._stack);
+        Main.layoutManager.overviewGroup.add_child(this._stack);
 
         this._coverPane.hide();
 
@@ -435,6 +433,7 @@ const Overview = new Lang.Class({
 
     beginItemDrag: function(source) {
         this.emit('item-drag-begin');
+        this._inDrag = true;
     },
 
     cancelledItemDrag: function(source) {
@@ -443,10 +442,12 @@ const Overview = new Lang.Class({
 
     endItemDrag: function(source) {
         this.emit('item-drag-end');
+        this._inDrag = false;
     },
 
     beginWindowDrag: function(source) {
         this.emit('window-drag-begin');
+        this._inDrag = true;
     },
 
     cancelledWindowDrag: function(source) {
@@ -455,24 +456,31 @@ const Overview = new Lang.Class({
 
     endWindowDrag: function(source) {
         this.emit('window-drag-end');
+        this._inDrag = false;
     },
 
     // show:
     //
     // Animates the overview visible and grabs mouse and keyboard input
-    show : function() {
+    show: function() {
         if (this.isDummy)
             return;
         if (this._shown)
             return;
         this._shown = true;
 
-        if (!this._syncInputMode())
+        if (!this._syncGrab())
             return;
 
+        Main.layoutManager.showOverview();
         this._animateVisible();
     },
 
+    focusSearch: function() {
+        this.show();
+        this._searchEntry.grab_key_focus();
+    },
+
     fadeInDesktop: function() {
             this._desktopFade.opacity = 0;
             this._desktopFade.show();
@@ -514,8 +522,6 @@ const Overview = new Lang.Class({
         //
         // Disable unredirection while in the overview
         Meta.disable_unredirect_for_screen(global.screen);
-        this._stack.show();
-        this._backgroundGroup.show();
         this._viewSelector.show();
 
         this._stack.opacity = 0;
@@ -556,7 +562,7 @@ const Overview = new Lang.Class({
         this._animateNotVisible();
 
         this._shown = false;
-        this._syncInputMode();
+        this._syncGrab();
     },
 
     toggle: function() {
@@ -578,6 +584,8 @@ const Overview = new Lang.Class({
     shouldToggleByCornerOrButton: function() {
         if (this.animationInProgress)
             return false;
+        if (this._inDrag)
+            return false;
         if (this._activationTime == 0 || Date.now() / 1000 - this._activationTime > OVERVIEW_ACTIVATION_TIMEOUT)
             return true;
         return false;
@@ -585,8 +593,8 @@ const Overview = new Lang.Class({
 
     //// Private methods ////
 
-    _syncInputMode: function() {
-        // We delay input mode changes during animation so that when removing the
+    _syncGrab: function() {
+        // We delay grab changes during animation so that when removing the
         // overview we don't have a problem with the release of a press/release
         // going to an application.
         if (this.animationInProgress)
@@ -604,16 +612,12 @@ const Overview = new Lang.Class({
                         return false;
                     }
                 }
-            } else {
-                global.stage_input_mode = Shell.StageInputMode.FULLSCREEN;
             }
         } else {
             if (this._modal) {
                 Main.popModal(this._overview);
                 this._modal = false;
             }
-            else if (global.stage_input_mode == Shell.StageInputMode.FULLSCREEN)
-                global.stage_input_mode = Shell.StageInputMode.NORMAL;
         }
         return true;
     },
@@ -652,7 +656,7 @@ const Overview = new Lang.Class({
         if (!this._shown)
             this._animateNotVisible();
 
-        this._syncInputMode();
+        this._syncGrab();
         global.sync_pointer();
     },
 
@@ -662,20 +666,19 @@ const Overview = new Lang.Class({
 
         this._viewSelector.hide();
         this._desktopFade.hide();
-        this._backgroundGroup.hide();
-        this._stack.hide();
+        this._coverPane.hide();
 
         this.visible = false;
         this.animationInProgress = false;
 
-        this._coverPane.hide();
-
         this.emit('hidden');
         // Handle any calls to show* while we were hiding
         if (this._shown)
             this._animateVisible();
+        else
+            Main.layoutManager.hideOverview();
 
-        this._syncInputMode();
+        this._syncGrab();
 
         // Fake a pointer event if requested
         if (this._needsFakePointerEvent) {
