@@ -393,12 +393,19 @@ const PopupSeparatorMenuItem = new Lang.Class({
     Name: 'PopupSeparatorMenuItem',
     Extends: PopupBaseMenuItem,
 
-    _init: function () {
+    _init: function (text) {
         this.parent({ reactive: false,
                       can_focus: false});
 
+        this._box = new St.BoxLayout();
+        this.addActor(this._box, { span: -1, expand: true });
+
+        this.label = new St.Label({ text: text || '' });
+        this._box.add(this.label);
+        this.actor.label_actor = this.label;
+
         this._separator = new Separator.HorizontalSeparator({ style_class: 'popup-separator-menu-item' });
-        this.addActor(this._separator.actor, { span: -1, expand: true });
+        this._box.add(this._separator.actor, { expand: true });
     }
 });
 
@@ -544,6 +551,10 @@ const PopupSliderMenuItem = new Lang.Class({
 
         let handleRadius = themeNode.get_length('-slider-handle-radius');
 
+        let handleBorderWidth = themeNode.get_length('-slider-handle-border-width');
+        let [hasHandleColor, handleBorderColor] =
+            themeNode.lookup_color('-slider-handle-border-color', false);
+
         let sliderWidth = width - 2 * handleRadius;
         let sliderHeight = themeNode.get_length('-slider-height');
 
@@ -595,7 +606,16 @@ const PopupSliderMenuItem = new Lang.Class({
             color.blue / 255,
             color.alpha / 255);
         cr.arc(handleX, handleY, handleRadius, 0, 2 * Math.PI);
-        cr.fill();
+        cr.fillPreserve();
+        if (hasHandleColor && handleBorderWidth) {
+          cr.setSourceRGBA(
+              handleBorderColor.red / 255,
+              handleBorderColor.green / 255,
+              handleBorderColor.blue / 255,
+              handleBorderColor.alpha / 255);
+          cr.setLineWidth(handleBorderWidth);
+          cr.stroke();
+        }
         cr.$dispose();
     },
 
@@ -983,6 +1003,9 @@ const PopupMenuBase = new Lang.Class({
     },
 
     _updateSeparatorVisibility: function(menuItem) {
+        if (menuItem.label.text)
+            return;
+
         let children = this.box.get_children();
 
         let index = children.indexOf(menuItem.actor);
@@ -1506,15 +1529,30 @@ const PopupSubMenuMenuItem = new Lang.Class({
         this.parent();
     },
 
+    setSubmenuShown: function(open) {
+        if (open)
+            this.menu.open(BoxPointer.PopupAnimation.FULL);
+        else
+            this.menu.close(BoxPointer.PopupAnimation.FULL);
+    },
+
+    _setOpenState: function(open) {
+        this.setSubmenuShown(open);
+    },
+
+    _getOpenState: function() {
+        return this.menu.isOpen;
+    },
+
     _onKeyPressEvent: function(actor, event) {
         let symbol = event.get_key_symbol();
 
         if (symbol == Clutter.KEY_Right) {
-            this.menu.open(BoxPointer.PopupAnimation.FULL);
+            this._setOpenState(true);
             this.menu.actor.navigate_focus(null, Gtk.DirectionType.DOWN, false);
             return true;
-        } else if (symbol == Clutter.KEY_Left && this.menu.isOpen) {
-            this.menu.close();
+        } else if (symbol == Clutter.KEY_Left && this._getOpenState()) {
+            this._setOpenState(false);
             return true;
         }
 
@@ -1522,11 +1560,11 @@ const PopupSubMenuMenuItem = new Lang.Class({
     },
 
     activate: function(event) {
-        this.menu.open(BoxPointer.PopupAnimation.FULL);
+        this._setOpenState(true);
     },
 
     _onButtonReleaseEvent: function(actor) {
-        this.menu.toggle();
+        this._setOpenState(!this._getOpenState());
     }
 });
 
@@ -1772,205 +1810,6 @@ const PopupComboBoxMenuItem = new Lang.Class({
     }
 });
 
-/**
- * RemoteMenu:
- *
- * A PopupMenu that tracks a GMenuModel and shows its actions
- * (exposed by GApplication/GActionGroup)
- */
-const RemoteMenu = new Lang.Class({
-    Name: 'RemoteMenu',
-    Extends: PopupMenu,
-
-    _init: function(sourceActor, model, actionGroup) {
-        this.parent(sourceActor, 0.0, St.Side.TOP);
-
-        this.model = model;
-        this.actionGroup = actionGroup;
-
-        this._actions = {};
-        this._trackMenu(model, this);
-
-        this._actionStateChangeId = this.actionGroup.connect('action-state-changed', Lang.bind(this, this._actionStateChanged));
-        this._actionEnableChangeId = this.actionGroup.connect('action-enabled-changed', Lang.bind(this, this._actionEnabledChanged));
-    },
-
-    destroy: function() {
-        if (this._actionStateChangeId) {
-            this.actionGroup.disconnect(this._actionStateChangeId);
-            this._actionStateChangeId = 0;
-        }
-
-        if (this._actionEnableChangeId) {
-            this.actionGroup.disconnect(this._actionEnableChangeId);
-            this._actionEnableChangeId = 0;
-        }
-
-        this.parent();
-    },
-
-    _actionAdded: function(model, item, index) {
-        let action_id = item.action_id;
-
-        if (!this._actions[action_id])
-            this._actions[action_id] = { enabled: this.actionGroup.get_action_enabled(action_id),
-                                         state: this.actionGroup.get_action_state(action_id),
-                                         items: [ ],
-                                       };
-        let action = this._actions[action_id];
-        let target, destroyId, specificSignalId;
-
-        if (action.state) {
-            // Docs have get_state_hint(), except that the DBus protocol
-            // has no provision for it (so ShellApp does not implement it,
-            // and neither GApplication), and g_action_get_state_hint()
-            // always returns null
-            // Funny :)
-
-            switch (String.fromCharCode(action.state.classify())) {
-            case 'b':
-                action.items.push(item);
-                item.setOrnament(action.state.get_boolean() ?
-                                 Ornament.CHECK : Ornament.NONE);
-                specificSignalId = item.connect('activate', Lang.bind(this, function(item) {
-                    this.actionGroup.activate_action(action_id, null);
-                }));
-                break;
-            case 's':
-                action.items.push(item);
-                item._remoteTarget = model.get_item_attribute_value(index, Gio.MENU_ATTRIBUTE_TARGET, null).deep_unpack();
-                item.setOrnament(action.state.deep_unpack() == item._remoteTarget ?
-                                 Ornament.DOT : Ornament.NONE);
-                specificSignalId = item.connect('activate', Lang.bind(this, function(item) {
-                    this.actionGroup.activate_action(action_id, GLib.Variant.new_string(item._remoteTarget));
-                }));
-                break;
-            default:
-                log('Action "%s" has state of type %s, which is not supported'.format(action_id, action.state.get_type_string()));
-                return;
-            }
-        } else {
-            target = model.get_item_attribute_value(index, Gio.MENU_ATTRIBUTE_TARGET, null);
-            action.items.push(item);
-            specificSignalId = item.connect('activate', Lang.bind(this, function() {
-                this.actionGroup.activate_action(action_id, target);
-            }));
-        }
-
-        item.actor.reactive = item.actor.can_focus = action.enabled;
-
-        destroyId = item.connect('destroy', Lang.bind(this, function() {
-            item.disconnect(destroyId);
-            item.disconnect(specificSignalId);
-
-            let pos = action.items.indexOf(item);
-            if (pos != -1)
-                action.items.splice(pos, 1);
-        }));
-    },
-
-    _trackMenu: function(model, item) {
-        item._tracker = Shell.MenuTracker.new(model,
-                                              null, /* action namespace */
-                                              Lang.bind(this, this._insertItem, item),
-                                              Lang.bind(this, this._removeItem, item));
-
-        item.connect('destroy', function() {
-            item._tracker.destroy();
-            item._tracker = null;
-        });
-    },
-
-    _createMenuItem: function(model, index) {
-        let labelValue = model.get_item_attribute_value(index, Gio.MENU_ATTRIBUTE_LABEL, null);
-        let label = labelValue ? labelValue.deep_unpack() : '';
-        // remove all underscores that are not followed by another underscore
-        label = label.replace(/_([^_])/, '$1');
-
-        let submenuModel = model.get_item_link(index, Gio.MENU_LINK_SUBMENU);
-        if (submenuModel) {
-            let item = new PopupSubMenuMenuItem(label);
-            this._trackMenu(submenuModel, item.menu);
-            return item;
-        }
-
-        let item = new PopupMenuItem(label);
-        let action_id = model.get_item_attribute_value(index, Gio.MENU_ATTRIBUTE_ACTION, null).deep_unpack();
-        item.actor.can_focus = item.actor.reactive = false;
-
-        item.action_id = action_id;
-
-        if (this.actionGroup.has_action(action_id)) {
-            this._actionAdded(model, item, index);
-            return item;
-        }
-
-        let signalId = this.actionGroup.connect('action-added', Lang.bind(this, function(actionGroup, actionName) {
-            actionGroup.disconnect(signalId);
-            if (this._actions[actionName]) return;
-
-            this._actionAdded(model, item, index);
-        }));
-
-        return item;
-    },
-
-    _actionStateChanged: function(actionGroup, action_id) {
-        let action = this._actions[action_id];
-        if (!action)
-            return;
-
-        action.state = actionGroup.get_action_state(action_id);
-        if (action.items.length) {
-            switch (String.fromCharCode(action.state.classify())) {
-            case 'b':
-                for (let i = 0; i < action.items.length; i++)
-                    action.items[i].setOrnament(action.state.get_boolean() ?
-                                                Ornament.CHECK : Ornament.NONE);
-                break;
-            case 'd':
-                for (let i = 0; i < action.items.length; i++)
-                    action.items[i].setValue(action.state.get_double());
-                break;
-            case 's':
-                for (let i = 0; i < action.items.length; i++)
-                    action.items[i].setOrnament(action.items[i]._remoteTarget == action.state.deep_unpack() ?
-                                                Ornament.DOT : Ornament.NONE);
-            }
-        }
-    },
-
-    _actionEnabledChanged: function(actionGroup, action_id) {
-        let action = this._actions[action_id];
-        if (!action)
-            return;
-
-        action.enabled = actionGroup.get_action_enabled(action_id);
-        if (action.items.length) {
-            for (let i = 0; i < action.items.length; i++) {
-                let item = action.items[i];
-                item.actor.reactive = item.actor.can_focus = action.enabled;
-            }
-        }
-    },
-
-    _insertItem: function(position, model, item_index, action_namespace, is_separator, target) {
-        let item;
-
-        if (is_separator)
-            item = new PopupSeparatorMenuItem();
-        else
-            item = this._createMenuItem(model, item_index);
-
-        target.addMenuItem(item, position);
-    },
-
-    _removeItem: function(position, target) {
-        let items = target._getMenuItems();
-        items[position].destroy();
-    },
-});
-
 /* Basic implementation of a menu manager.
  * Call addMenu to add menus
  */
@@ -2049,7 +1888,7 @@ const PopupMenuManager = new Lang.Class({
 
     _onMenuOpenState: function(menu, open) {
         if (open) {
-            if (this.activeMenu)
+            if (this.activeMenu && !this.activeMenu.isChildMenu(menu))
                 this.activeMenu.close(BoxPointer.PopupAnimation.FADE);
             this._grabHelper.grab({ actor: menu.actor, modal: true, focus: menu.sourceActor,
                                     onUngrab: Lang.bind(this, this._closeMenu, menu) });
