@@ -30,6 +30,7 @@ const SCREENSAVER_SCHEMA = 'org.gnome.desktop.screensaver';
 const LOCK_ENABLED_KEY = 'lock-enabled';
 const LOCK_DELAY_KEY = 'lock-delay';
 
+const LOCKED_STATE_STR = 'screenShield.locked';
 // fraction of screen height the arrow must reach before completing
 // the slide up automatically
 const ARROW_DRAG_THRESHOLD = 0.1;
@@ -214,6 +215,7 @@ const NotificationsBox = new Lang.Class({
 
         if (musicNotification != null &&
             this._musicBin.child == null) {
+            musicNotification.acknowledged = true;
             if (musicNotification.actor.get_parent() != null)
                 musicNotification.actor.get_parent().remove_actor(musicNotification.actor);
             this._musicBin.child = musicNotification.actor;
@@ -246,6 +248,7 @@ const NotificationsBox = new Lang.Class({
             sourceCountChangedId: 0,
             sourceTitleChangedId: 0,
             sourceUpdatedId: 0,
+            sourceNotifyId: 0,
             musicNotification: null,
             sourceBox: null,
             titleLabel: null,
@@ -256,6 +259,12 @@ const NotificationsBox = new Lang.Class({
         this._showSource(source, obj, obj.sourceBox);
         this._notificationBox.add(obj.sourceBox, { x_fill: false, x_align: St.Align.START });
 
+        if (obj.musicNotification) {
+            obj.sourceNotifyId = source.connect('notify', Lang.bind(this, function(source, notification) {
+                notification.acknowledged = true;
+            }));
+        }
+
         obj.sourceCountChangedId = source.connect('count-updated', Lang.bind(this, function(source) {
             this._countChanged(source, obj);
         }));
@@ -336,6 +345,8 @@ const NotificationsBox = new Lang.Class({
         if (obj.musicNotification) {
             this._musicBin.child = null;
             obj.musicNotification = null;
+
+            source.disconnect(obj.sourceNotifyId);
         }
 
         source.disconnect(obj.sourceDestroyId);
@@ -709,8 +720,6 @@ const ScreenShield = new Lang.Class({
     },
 
     _onDragEnd: function(action, actor, eventX, eventY, modifiers) {
-        if (this._lockScreenState != MessageTray.State.HIDING)
-            return;
         if (this._lockScreenGroup.y < -(ARROW_DRAG_THRESHOLD * global.stage.height)) {
             // Complete motion automatically
 	    let [velocity, velocityX, velocityY] = this._dragAction.get_velocity(0);
@@ -1125,6 +1134,14 @@ const ScreenShield = new Lang.Class({
         if (Main.sessionMode.currentMode == 'unlock-dialog')
             Main.sessionMode.popMode('unlock-dialog');
 
+        if (this._dialog && !this._isGreeter)
+            this._dialog.popModal();
+
+        if (this._isModal) {
+            Main.popModal(this.actor);
+            this._isModal = false;
+        }
+
         Tweener.addTween(this._lockDialogGroup, {
             scale_x: 0,
             scale_y: 0,
@@ -1142,12 +1159,6 @@ const ScreenShield = new Lang.Class({
         }
 
         this._lightbox.hide();
-
-        if (this._isModal) {
-            Main.popModal(this.actor);
-            this._isModal = false;
-        }
-
         this.actor.hide();
 
         if (this._becameActiveId != 0) {
@@ -1165,6 +1176,7 @@ const ScreenShield = new Lang.Class({
         this._isLocked = false;
         this.emit('active-changed');
         this.emit('locked-changed');
+        global.set_runtime_state(LOCKED_STATE_STR, null);
     },
 
     activate: function(animate) {
@@ -1181,6 +1193,7 @@ const ScreenShield = new Lang.Class({
         }
 
         this._resetLockScreen(animate, animate);
+        global.set_runtime_state(LOCKED_STATE_STR, GLib.Variant.new('b', true));
 
         // We used to set isActive and emit active-changed here,
         // but now we do that from lockScreenShown, which means
@@ -1202,16 +1215,20 @@ const ScreenShield = new Lang.Class({
             return;
         }
 
-        // Clear the clipboard - otherwise, its contents may be leaked
-        // to unauthorized parties by pasting into the unlock dialog's
-        // password entry and unmasking the entry
-        St.Clipboard.get_default().set_text(St.ClipboardType.CLIPBOARD, '');
-        St.Clipboard.get_default().set_text(St.ClipboardType.PRIMARY, '');
-
         this._isLocked = true;
         this.activate(animate);
 
         this.emit('locked-changed');
     },
+
+    // If the previous shell crashed, and gnome-session restarted us, then re-lock
+    lockIfWasLocked: function() {
+        let wasLocked = global.get_runtime_state('b', LOCKED_STATE_STR);
+        if (wasLocked === null)
+            return;
+        Meta.later_add(Meta.LaterType.BEFORE_REDRAW, Lang.bind(this, function() {
+            this.lock(false);
+        }));
+    }
 });
 Signals.addSignalMethods(ScreenShield.prototype);
