@@ -45,7 +45,7 @@ const _modes = {
         unlockDialog: imports.gdm.loginDialog.LoginDialog,
         components: ['polkitAgent'],
         panel: {
-            left: ['logo'],
+            left: [],
             center: ['dateMenu'],
             right: ['a11yGreeter', 'display', 'keyboard',
                     'volume', 'battery', 'powerMenu']
@@ -78,17 +78,6 @@ const _modes = {
         panelStyle: 'unlock-screen'
     },
 
-    'initial-setup': {
-        hasWindows: true,
-        isPrimary: true,
-        components: ['networkAgent', 'keyring'],
-        panel: {
-            left: [],
-            center: [],
-            right: ['a11yGreeter', 'keyboard', 'volume', 'battery']
-        }
-    },
-
     'user': {
         hasOverview: true,
         showCalendarEvents: true,
@@ -195,6 +184,10 @@ const SessionMode = new Lang.Class({
         return this._modeStack[this._modeStack.length - 1];
     },
 
+    get allowScreencast() {
+        return this.components.indexOf('recorder') != -1;
+    },
+
     _sync: function() {
         let params = this._modes[this.currentMode];
         let defaults;
