@@ -69,13 +69,6 @@ function signalToIcon(value) {
     return 'none';
 }
 
-// shared between NMNetworkMenuItem and NMDeviceWireless
-function sortAccessPoints(accessPoints) {
-    return accessPoints.sort(function (one, two) {
-        return two.strength - one.strength;
-    });
-}
-
 function ssidToLabel(ssid) {
     let label = NetworkManager.utils_ssid_to_utf8(ssid);
     if (!label)
@@ -180,22 +173,7 @@ const NMConnectionBased = new Lang.Class({
 
     _init: function(connections) {
         this._connections = [ ];
-        for (let i = 0; i < connections.length; i++) {
-            if (!connections[i].get_uuid())
-                continue;
-            if (!this.connectionValid(connections[i]))
-                continue;
-            // record the connection
-            let obj = {
-                connection: connections[i],
-                name: connections[i].get_id(),
-                uuid: connections[i].get_uuid(),
-                timestamp: connections[i]._timestamp,
-                item: null,
-            };
-            this._connections.push(obj);
-        }
-        this._connections.sort(this._connectionSortFunction);
+        connections.forEach(Lang.bind(this, this.checkConnection));
     },
 
     checkConnection: function(connection) {
@@ -786,58 +764,12 @@ const NMDeviceWireless = new Lang.Class({
         this._overflowItem = null;
         this._networks = [ ];
 
-        // breaking the layers with this, but cannot call
-        // this.connectionValid until I have a device
-        this.device = device;
+        this.parent(client, device, connections);
 
-        let validConnections = connections.filter(Lang.bind(this, function(connection) {
-            return this.connectionValid(connection);
-        }));
         let accessPoints = device.get_access_points() || [ ];
-        for (let i = 0; i < accessPoints.length; i++) {
-            // Access points are grouped by network
-            let ap = accessPoints[i];
-
-            if (ap.get_ssid() == null) {
-                // hidden access point cannot be added, we need to know
-                // the SSID and security details to connect
-                // nevertheless, the access point can acquire a SSID when
-                // NetworkManager connects to it (via nmcli or the control-center)
-                ap._notifySsidId = ap.connect('notify::ssid', Lang.bind(this, this._notifySsidCb));
-                continue;
-            }
-
-            let pos = this._findNetwork(ap);
-            let network;
-            if (pos != -1) {
-                network = this._networks[pos];
-                network.accessPoints.push(ap);
-            } else {
-                network = { ssid: ap.get_ssid(),
-                            mode: ap.mode,
-                            security: this._getApSecurityType(ap),
-                            connections: [ ],
-                            item: null,
-                            accessPoints: [ ap ]
-                      };
-                network.ssidText = ssidToLabel(network.ssid);
-                this._networks.push(network);
-            }
-            ap._updateId = ap.connect('notify::strength', Lang.bind(this, this._onApStrengthChanged));
-
-            // Check if some connection is valid for this AP
-            for (let j = 0; j < validConnections.length; j++) {
-                let connection = validConnections[j];
-                if (ap.connection_valid(connection) &&
-                    network.connections.indexOf(connection) == -1) {
-                    network.connections.push(connection);
-                }
-            }
-        }
-
-        // Sort APs within each network by strength
-        for (let i = 0; i < this._networks.length; i++)
-            sortAccessPoints(this._networks[i].accessPoints);
+        accessPoints.forEach(Lang.bind(this, function(ap) {
+            this._accessPointAdded(this.device, ap);
+        }));
 
         this._activeApChanged();
         this._networks.sort(this._networkSortFunction);
@@ -845,8 +777,6 @@ const NMDeviceWireless = new Lang.Class({
         this._apChangedId = device.connect('notify::active-access-point', Lang.bind(this, this._activeApChanged));
         this._apAddedId = device.connect('access-point-added', Lang.bind(this, this._accessPointAdded));
         this._apRemovedId = device.connect('access-point-removed', Lang.bind(this, this._accessPointRemoved));
-
-        this.parent(client, device, validConnections);
     },
 
     destroy: function() {
@@ -1389,18 +1319,14 @@ const NMVPNSection = new Lang.Class({
     Extends: NMConnectionBased,
     category: NMConnectionCategory.VPN,
 
-    _init: function(client, connections) {
-        this.parent(connections);
+    _init: function(client) {
+        this.parent([]);
         this._client = client;
 
         this.section = new PopupMenu.PopupMenuSection();
         this._deferredWorkId = Main.initializeDeferredWork(this.section.actor, Lang.bind(this, this._createSection));
     },
 
-    get empty() {
-        return this._connections.length == 0;
-    },
-
     connectionValid: function(connection) {
         // filtering is done by NMApplet code
         return true;
@@ -1650,7 +1576,7 @@ const NMApplet = new Lang.Class({
         this.menu.addMenuItem(this._devices.wwan.section);
         this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
 
-        this._vpnSection = new NMVPNSection(this._client, this._connections);
+        this._vpnSection = new NMVPNSection(this._client);
         this._vpnSection.connect('activation-failed', Lang.bind(this, this._onActivationFailed));
         this.menu.addMenuItem(this._vpnSection.section);
         this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
@@ -1904,6 +1830,7 @@ const NMApplet = new Lang.Class({
         let default_ip4 = null;
         let default_ip6 = null;
         let active_vpn = null;
+        let active_any = null;
         for (let i = 0; i < this._activeConnections.length; i++) {
             let a = this._activeConnections[i];
 
@@ -1934,24 +1861,21 @@ const NMApplet = new Lang.Class({
             if (a.default6)
                 default_ip6 = a;
 
-            if (a._type == 'vpn')
-                active_vpn = a;
-            else if (a.state == NetworkManager.ActiveConnectionState.ACTIVATING)
+            if (a.state == NetworkManager.ActiveConnectionState.ACTIVATING)
                 activating = a;
+            else if (a.state == NetworkManager.ActiveConnectionState.ACTIVATED)
+                active_any = a;
+
+            if (a._type == 'vpn' &&
+                (a.state == NetworkManager.ActiveConnectionState.ACTIVATING ||
+                 a.state == NetworkManager.ActiveConnectionState.ACTIVATED))
+                active_vpn = a;
 
             if (!a._primaryDevice) {
-                if (a._type != NetworkManager.SETTING_VPN_SETTING_NAME) {
-                    // find a good device to be considered primary
-                    a._primaryDevice = null;
-                    let devices = a.get_devices() || [];
-                    for (let j = 0; j < devices.length; j++) {
-                        let d = devices[j];
-                        if (d._delegate) {
-                            a._primaryDevice = d._delegate;
-                            break;
-                        }
-                    }
-                } else
+                if (a._type != NetworkManager.SETTING_VPN_SETTING_NAME)
+                    // This list is guaranteed to have one device in it.
+                    a._primaryDevice = a.get_devices()[0]._delegate;
+                else
                     a._primaryDevice = this._vpnSection;
 
                 if (a._primaryDevice)
@@ -1965,7 +1889,7 @@ const NMApplet = new Lang.Class({
             }
         }
 
-        this._mainConnection = activating || default_ip4 || default_ip6 || this._activeConnections[0] || null;
+        this._mainConnection = activating || default_ip4 || default_ip6 || active_any || null;
         this._vpnConnection = active_vpn;
     },
 
