@@ -453,6 +453,10 @@ const WindowOverlay = new Lang.Class({
         metaWindow.delete(global.get_current_time());
     },
 
+    _windowCanClose: function() {
+        return this._windowClone.metaWindow.can_close();
+    },
+
     _onWindowAdded: function(workspace, win) {
         let metaWindow = this._windowClone.metaWindow;
 
@@ -488,12 +492,14 @@ const WindowOverlay = new Lang.Class({
     _animateVisible: function() {
         this._parentActor.raise_top();
 
-        this.closeButton.show();
-        this.closeButton.opacity = 0;
-        Tweener.addTween(this.closeButton,
-                         { opacity: 255,
-                           time: CLOSE_BUTTON_FADE_TIME,
-                           transition: 'easeOutQuad' });
+        if (this._windowCanClose()) {
+            this.closeButton.show();
+            this.closeButton.opacity = 0;
+            Tweener.addTween(this.closeButton,
+                             { opacity: 255,
+                               time: CLOSE_BUTTON_FADE_TIME,
+                               transition: 'easeOutQuad' });
+        }
 
         this.border.show();
         this.border.opacity = 0;
@@ -936,7 +942,7 @@ const Workspace = new Lang.Class({
             this.actor.add_style_class_name('external-monitor');
         this.actor.set_size(0, 0);
 
-        this._dropRect = new Clutter.Rectangle({ opacity: 0 });
+        this._dropRect = new Clutter.Actor({ opacity: 0 });
         this._dropRect._delegate = this;
 
         this.actor.add_actor(this._dropRect);
@@ -983,10 +989,17 @@ const Workspace = new Lang.Class({
     setActualGeometry: function(geom) {
         this._actualGeometry = geom;
 
-        Meta.later_add(Meta.LaterType.BEFORE_REDRAW, Lang.bind(this, function() {
+        if (this._actualGeometryLater)
+            return;
+
+        this._actualGeometryLater = Meta.later_add(Meta.LaterType.BEFORE_REDRAW, Lang.bind(this, function() {
+            let geom = this._actualGeometry;
+
             this._dropRect.set_position(geom.x, geom.y);
             this._dropRect.set_size(geom.width, geom.height);
             this._updateWindowPositions(WindowPositionFlags.NONE);
+
+            this._actualGeometryLater = 0;
             return false;
         }));
     },
