@@ -100,19 +100,11 @@ struct _StThemeNode {
   guint text_shadow_computed : 1;
   guint link_type : 2;
 
-  /* Graphics state */
-  float alloc_width;
-  float alloc_height;
-
-  CoglHandle background_shadow_material;
-  CoglHandle box_shadow_material;
-  CoglHandle background_texture;
-  CoglHandle background_material;
   CoglHandle border_slices_texture;
   CoglHandle border_slices_material;
-  CoglHandle prerendered_texture;
-  CoglHandle prerendered_material;
-  CoglHandle corner_material[4];
+  CoglHandle background_texture;
+  CoglHandle background_material;
+  CoglHandle background_shadow_material;
 };
 
 struct _StThemeNodeClass {
@@ -123,9 +115,6 @@ struct _StThemeNodeClass {
 void _st_theme_node_ensure_background (StThemeNode *node);
 void _st_theme_node_ensure_geometry (StThemeNode *node);
 
-void _st_theme_node_init_drawing_state (StThemeNode *node);
-void _st_theme_node_free_drawing_state (StThemeNode *node);
-
 G_END_DECLS
 
 #endif /* __ST_THEME_NODE_PRIVATE_H__ */
